--------------------------------------------------------------------------------------------------------------------
--[[
	LuAstar - Pathfinding A-Star Library For Lua
	Demo test File
    Copyright (C) 2011.
	Written by Roland Yonaba - E-mail: roland[dot]yonaba[at]gmail[dot]com
--]]


-- Colors
local WHITE = {255,255,255}
local YELLOW = {255,255,0}
local BLUE = {0,0,255}
local GREEN = {0,255,0}

-- Constants
local W = love.graphics.getWidth()  -- Window width
local H = love.graphics.getHeight() -- Window height
local CELL_SIZE_X = 30  -- cell Width
local CELL_SIZE_Y = 30  -- cell height

-- Map : 0 refers to a walkable location
local grid = { }
	for i = 1,math.floor(H/CELL_SIZE_Y),1 do
	grid[i] = {}
		for j = 1,math.floor(W/CELL_SIZE_X),1 do
		grid[i][j] = 0
		end
	end

-- Draws unwalkable cells
local function drawBlocks(m)
	for j in ipairs(m) do
		for i in ipairs(m[j]) do
			if m[j][i] == 1 then
			love.graphics.setColor(BLUE)
			love.graphics.rectangle('fill',(i-1)*CELL_SIZE_X,(j-1)*CELL_SIZE_Y,CELL_SIZE_X,CELL_SIZE_Y)
			end
		end
	end
end

-- Draws the entire grid
local function drawGrid(m)
	local gridSizeX = #m[(next(m))]
	local gridSizeY = #m
	love.graphics.setColor(WHITE)
		for x = 1,gridSizeX do
		love.graphics.line(x*CELL_SIZE_X,0,x*CELL_SIZE_X,H)
		end
		for y=1,gridSizeY do
		love.graphics.line(0,y*CELL_SIZE_Y,W,y*CELL_SIZE_Y)
		end
	drawBlocks(m)
end

--  Displays the cell on which the mouse is
function printDebug()
	local x,y = love.mouse.getPosition()
	local gX = math.floor(x/CELL_SIZE_X)+1
	local gY = math.floor(y/CELL_SIZE_Y)+1
	love.graphics.setColor(YELLOW)
	love.graphics.print('CellX : '..gX,W-150,20)
	love.graphics.print('CellY : '..gY,W-150,30)
	love.graphics.print('Right Click : Set Obstacles',W-150,40)
	love.graphics.print('Left Click : Moves unit',W-150,50)
end

-- a simple unit
local unit = { x = 1, y = 1, w = CELL_SIZE_X, h = CELL_SIZE_Y , uX = 1,uY = 1,
				path = {}, pathLen = 0, tx = 0, ty = 0, move = false, speed = 0.5,
				}
	-- draws the unit			
	function unit.draw()
		love.graphics.setColor(GREEN)
		love.graphics.rectangle('fill',unit.x,unit.y,CELL_SIZE_X,CELL_SIZE_Y)
	end
	
	
local Astar = require 'Lib.Astar'  -- load library, returns a set of functions in a 'Astar' table
Astar(grid)   --Inits the library
Astar:setObstValue(1)  -- Optional: set value of unwalkabale locations (default OBST_VALUE is 1)
Astar:disableDiagonalMove()  -- Optional: disable diagonal moves (diagonal moves are disabled by default, but they can be enabled)
Astar:setSearchDepth(0)  -- Set a search depth to look around the final location if unreachable (default search depth is 0)

-- update callback
function love.update(dt)
	-- moves the unit
	if unit.move then
		if unit.x < unit.tx then
		unit.x = unit.x + unit.speed*(dt*1000/3)
		elseif unit.x > unit.tx then
		unit.x = unit.x - unit.speed*(dt*1000/3)
		end
		if unit.y < unit.ty then
		unit.y = unit.y + unit.speed*(dt*1000/3)
		elseif unit.y > unit.ty then
		unit.y = unit.y - unit.speed*(dt*1000/3)
		end
		-- temporary target reaching
		if (math.abs(unit.x-unit.tx) <= 2) and (math.abs(unit.y-unit.ty) <= 2) then
			if unit.cur < unit.pathLen then
				-- assign next temporary target
				unit.cur = unit.cur+1
				unit.tx = unit.path[unit.cur].x*CELL_SIZE_X-CELL_SIZE_X
				unit.ty = unit.path[unit.cur].y*CELL_SIZE_Y-CELL_SIZE_Y
			else
				-- Stops move
				unit.move = false
				unit.x = unit.path[unit.pathLen].x*CELL_SIZE_X-CELL_SIZE_X
				unit.y = unit.path[unit.pathLen].y*CELL_SIZE_Y-CELL_SIZE_Y
				unit.cur = 0
				unit.pathLen = 0
				unit.path = {}
				unit.tx,unit.ty = 0,0
			end
		end
		
	end
end


love.graphics.setFont(10)  --sets internal font size
-- drawing callback
function love.draw()
	drawGrid(grid)  -- draws the map
	unit.draw()  -- draws the unit
	printDebug()  -- display debug infos
end

-- mousepressed callback
function love.mousepressed(x,y,button)
	-- On which cell is the mouse ?
	local gX = math.floor(x/CELL_SIZE_X)+1  
	local gY = math.floor(y/CELL_SIZE_Y)+1
	-- On which cell is the player ?
	unit.uX = math.floor(unit.x/CELL_SIZE_X)+1
	unit.uY = math.floor(unit.y/CELL_SIZE_Y)+1	
		-- Sets current cell walkable/unwalkable 
		if (button == 'r') and ((unit.uX ~= gX) or (unit.uY ~= gY)) then
		grid[gY][gX] = (grid[gY][gX]==0 and 1 or 0)
		end
		-- Assign new destination to the unit
		if button=='l' then
		Astar:setInitialNode(unit.uX,unit.uY)   -- sets initial cell
		Astar:setFinalNode(gX,gY) -- sets final cell
		unit.path = Astar:getPath()  -- computes and returns path using Astar algorithm
			if not unit.path then   --path not found ?
			Astar:reset()
			else
				-- start moving unit
				unit.pathLen = #unit.path
				unit.cur = 1
				unit.move = true
				unit.tx = unit.path[unit.cur].x*CELL_SIZE_X-CELL_SIZE_X
				unit.ty = unit.path[unit.cur].y*CELL_SIZE_Y-CELL_SIZE_Y				
			end
		end		
end
